/***************************************************************************
 *   Copyright (C) 2004 by Petric Frank                                    *
 *   pfrank@gmx.de                                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <iostream>
#include "function-factory.h"
#include "ctl-function.h"



void
program_version ()
{
  std::cout << "cvsfsctl " << VERSION << std::endl;
}



void
help ()
{
  program_version ();
  
  std::cerr << std::endl
            << "usage: cvsctl function [options]" << std::endl
	    << "  functions are:" << std::endl
	    << "    help <function> ... give help for function <function>." << std::endl
	    << "    getview <path> .... get actual view for path" << std::endl
	    << "    setview <path> .... set new view for path" << std::endl
	    << "    getlayer <file> ... retrieves the layer the file comes from" << std::endl
	    << "    getversion <file> . version of the file given" << std::endl
	    << "    cvslog <file> ..... obtain the output of the last CVS action" << std::endl
	    << "    checkout <file> ... checkout file from CVS server" << std::endl
	    << "    checkin <file> .... checkin file to CVS server" << std::endl
	    << "    add <file> ........ add file to CVS server" << std::endl
	    << std::endl;
}



int main(int argc, char *argv[])
{
  function_factory factory;

  if (argc < 2)
  {
    help ();
    
    return 1;
  }

  ctl_function *func = factory.generate (argv[1]);
  
  if (!func)
  {
    help ();
    
    return 2;
  }

  // execute the requested function
  bool success = func->execute (argc, argv);
  
  delete func;
  
  return success ? 0 : 3;
}
