/***************************************************************************
 *   Copyright (C) 2004 by Petric Frank                                    *
 *   pfrank@gmx.de                                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "setview-function.h"
#include <sys/xattr.h>
#include <errno.h>
#include <iostream>



setview_function::setview_function ()
: ctl_function ()
{
}



bool setview_function::execute (int &argc, char *argv[])
{
  if (argc < 4)
  {
    help ();
    
    return false;
  }

  std::string view = argv[3];

  if (argc > 4)
  {
    view += " ";
    view += argv[4];
  }

  if (argc > 5)
  {
    view += " ";
    view += argv[5];
  }

  int size = lsetxattr (argv[2], "view", view.c_str (), view.length () + 1, XATTR_REPLACE);
  
  if (size < 0)
  {
    int err = errno;
  
    std::cerr << "Error setting view." << std::endl
              << "  Code : " << err << " (" << strerror (err) << ")" << std::endl;

    return false;
  }

  return true;
}



void setview_function::help () const
{
  std::cerr << "Set the view for a cvsfs-fuse mountpoint." << std::endl
            << std::endl
	    << "Command:" << std::endl
	    << "  cvsfsctl setview <mountpoint> <repository> [<module> [<label>]]" << std::endl
	    << std::endl
	    << "<repository> is alike CVSROOT environment variable:" << std::endl
	    << "  [:<method>:][<userid>[:<passord>]@]<server>[:port]<cvsroot>" << std::endl
	    << std::endl
	    << "where" << std::endl
	    << "  <method> .... actually only 'pserver' is supported." << std::endl
	    << "  <userid> .... user ID to log into CVS server (default: 'anonymous')" << std::endl
	    << "  <password> .. password to log into CVS server (default: <empty>)" << std::endl
	    << "  <server> .... IP address or hostname of CVS server" << std::endl
	    << "  <port> ...... port for contacting the CVS server (default: 2401 for pserver)" << std::endl
	    << "  <module> .... the module of the repository (default: '.')" << std::endl
	    << "  <label> ..... the chekout label (default: 'HEAD')" << std::endl;
}
