/***************************************************************************
 *   Copyright (C) 2004 by Petric Frank                                    *
 *   pfrank@gmx.de                                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "cvs-command-add.h"
#include "cvs-session.h"
#include "cvs-connection.h"
#include "cvs-directory-entry.h"



cvs_command_add::cvs_command_add (const std::string &path,
                                  const std::string &comment, cvs_directory_entry &entry)
: cvs_command (), item_path (path), item_comment (comment), item_entry (entry)
{
  // strip version - if any
  std::string::size_type pos = item_path.find ("@@");
  if (pos != std::string::npos)
    item_path.erase (pos);

  // strip away leading slash - if any
  if ((item_path.length () > 0) && (item_path[0] == '/'))
    item_path.erase (0, 1);
}



bool cvs_command_add::run (cvs_session &session)
{
  // reset the log filename
  item_entry.set_logfilename ("");
  
  // store responses from CVS server
  if (establish_logging (session))
    item_entry.set_logfilename (filename);

  // send CVS preparation command
  if (!send_prepare_command (session))
    return false;

  // get response (ok) from CVS server
  if (!process_response (session))
    return false;

  // send CVS command
  if (!send_command (session))
    return false;

  // obtain response from CVS server
  if (!process_response (session))
    return false;

  // set version number
  if (item_entry.is_dir ())
    item_entry.set_version ("");
  else
    item_entry.set_version ("0");

  return true;
}



bool cvs_command_add::send_prepare_command (cvs_session &session)
{
  session.send () << "wrapper-sendme-rcsOptions" << std::endl;

  return true;
}



bool cvs_command_add::send_command (cvs_session &session)
{
  std::string::size_type pos;

  session.send () << "Argument -m" << std::endl;

  // send CVS comment
  if (item_comment.length () > 0)
  {
    std::string comment = item_comment;
    pos = comment.find ('\n');
    std::string remain;
    if (pos != std::string::npos)
    {
      remain = comment;
      remain.erase (0, pos + 1);
      comment.erase (pos);
    }
    session.send () << "Argument " << comment << std::endl;
  
    while (remain.length () != 0)
    {
      comment = remain;
      pos = comment.find ('\n');
      if (pos != std::string::npos)
      {
        remain.erase (0, pos + 1);
        comment.erase (pos);
      }
      else
        remain.clear ();
    
      session.send () << "Argumentx " << comment << std::endl;
    }
  }

  session.send () << "Argument --" << std::endl;

  std::string path = session.get_connection ().cvsroot () + "/" + item_path;
  pos = path.rfind ('/');

  if (item_entry.is_dir ())
  {
    std::string name = path;
    if (pos != std::string::npos)
      name.erase (0, pos + 1);

    session.send () << "Directory " << name << std::endl
		    << path << std::endl
		    << "Argument " << name << std::endl;
  }
  else
  {
    if (pos == std::string::npos)
      path.clear ();
    else
      path.erase (pos);

    session.send () << "Directory ." << std::endl
                    << path << std::endl
                    << "Is-modified " << item_entry.name () << std::endl
		    << "Argument " << item_entry.name () << std::endl;
  }

  session.send () << "add" << std::endl;

  return true;
}



bool cvs_command_add::process_line (cvs_session & session, const std::string & line)
{
  return true;
}
