/***************************************************************************
 *   Copyright (C) 2004 by Petric Frank                                    *
 *   pfrank@gmx.de                                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef __CVSFS_FUSE_H__
#define __CVSFS_FUSE_H__

#include "fuse-base.h"
#include <sys/types.h>
#include <string>
#include "view.h"



// forward reference
class fs_layer;



class cvsfs_fuse : public fuse_base
{
  public:
    cvsfs_fuse ();
    virtual ~cvsfs_fuse ();
  
    void set_uid (uid_t new_uid) { uid = new_uid; }
    void set_gid (gid_t new_gid) { gid = new_gid; }

    void shutdown ();

  protected:
    virtual bool run_prepare (int &, char *[]);
    virtual void help_options ();

  private:
    int change_view (const char *, size_t);
    int get_view (char *, size_t) const;
    int add_repository (const std::string &);
    int handle_response (const std::string &, char *, size_t);

    virtual int getattr (const char *, struct stat *);
    virtual int readlink (const char *, char *, size_t);
    virtual int getdir (const char *, fuse_dirh_t, fuse_dirfil_t);
    virtual int mknod (const char *, mode_t, dev_t);
    virtual int mkdir (const char *, mode_t);
    virtual int unlink (const char *);
    virtual int rmdir (const char *);
    virtual int symlink (const char *, const char *);
    virtual int rename (const char *, const char *);
    virtual int link (const char *, const char *);
    virtual int chmod (const char *, mode_t);
    virtual int chown (const char *, uid_t, gid_t);
    virtual int truncate (const char *, off_t);
    virtual int utime (const char *, struct utimbuf *);
    virtual int open (const char *, int);
    virtual int read (const char *, char *, size_t, off_t);
    virtual int write (const char *, const char *, size_t, off_t);
    virtual int statfs (const char *, struct statfs *);
    virtual int flush (const char *);
    virtual int release (const char *, int);
    virtual int fsync (const char *, int);
    virtual int setxattr (const char *, const char *, const char *, size_t, int);
    virtual int getxattr (const char *, const char *, char *, size_t);
    virtual int listxattr (const char *, char *, size_t);
    virtual int removexattr (const char *, const char *);
    
    uid_t	uid;
    gid_t	gid;
    fs_layer	*private_files;
    view	mount_view;
    std::string users_home;
    std::string cache_base;
    
    typedef std::list<std::string> project_list;
    
    project_list projects;
};



#endif
