/***************************************************************************
 *   Copyright (C) 2004 by Petric Frank                                    *
 *   pfrank@gmx.de                                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "fs-layer-private.h"
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/vfs.h>
#include <sys/xattr.h>
#include <dirent.h>
#include <utime.h>
#include <fcntl.h>
#include <unistd.h>



static char *LayerAttrName	= "layer";
static char *LayerAttrValue	= "Private";



fs_layer_private::fs_layer_private (const std::string & base)
: fs_layer (base)
{
}



directory *fs_layer_private::readdir (const std::string &path) const
{
  directory *dir = new directory ();
  std::string fullpath = cache_location + path;

  DIR *posix_dir = ::opendir (fullpath.c_str ());
  if (posix_dir != NULL)
  {
    struct dirent *entry;
    
    while ((entry = ::readdir (posix_dir)) != NULL)
    {
      dir->push_back (entry->d_name);
    }
      
    ::closedir (posix_dir);
  }

  return dir;
}



int fs_layer_private::mknod (const std::string &path, mode_t mode, dev_t rdev)
{
  int result = fs_layer::mknod (path, mode, rdev);
  
  // in case of a regular file - create the directory for it
  if ((result < 0) && S_ISREG (mode))
  {
    std::string dir = cache_location + path;
    dir.erase (dir.rfind ('/'));
    
    if (create_directory (dir, S_IRUSR | S_IWUSR | S_IXUSR))
      result = fs_layer::mknod (path, mode, rdev);
  }

  if (result < 0)
    return result;

  return 0;
}



int fs_layer_private::getxattr (const std::string &path, const std::string &name, char *value, size_t size) const
{
  if (name == LayerAttrName)
  {
    if (size == 0)
      return strlen (LayerAttrValue) + 1;

    if (size <= strlen (LayerAttrValue))
      return -ERANGE;
      
    strcpy (value, LayerAttrValue);
    
    return strlen (value) + 1;
  }

  if (name == "version")
  {
    if (size == 0)
      return 1;

    *value = '\0';

    return 1;
  }
  
  return fs_layer::getxattr (path, name.c_str (), value, size);
}
