/***************************************************************************
 *   Copyright (C) 2004 by Petric Frank                                    *
 *   pfrank@gmx.de                                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef __FS_LAYER_H__
#define __FS_LAYER_H__

#include <string>
#include "directory.h"

// forward references
class repository;



class fs_layer
{
  public:
    fs_layer (const std::string &);
    virtual ~fs_layer ();

    const std::string &get_basedir () const { return cache_location; }

    virtual repository *get_repository ();

    virtual directory *readdir (const std::string &) const = 0;
    virtual int getattr (const std::string &, struct stat *) const;
    virtual int readlink (const std::string &, char *, size_t) const;
    virtual int mknod (const std::string &, mode_t, dev_t);
    virtual int mkdir (const std::string &, mode_t);
    virtual int unlink (const std::string &);
    virtual int rmdir (const std::string &);
    virtual int symlink (const std::string &, const std::string &);
    virtual int rename (const std::string &, const std::string &);
    virtual int link (const std::string &, const std::string &);
    virtual int chmod (const std::string &, mode_t);
    virtual int chown (const std::string &, uid_t, gid_t);
    virtual int truncate (const std::string &, off_t);
    virtual int utime (const std::string &, struct utimbuf *);
    virtual int open (const std::string &, int);
    virtual int read (const std::string &, char *, size_t, off_t) const;
    virtual int write (const std::string &, const char *, size_t, off_t);
    virtual int statfs (const std::string &, struct statfs *) const;
    virtual int flush (const std::string &);
    virtual int release (const std::string &, int);
    virtual int fsync (const std::string &, int);
    virtual int setxattr (const std::string &, const std::string &, const std::string &, size_t, int);
    virtual int getxattr (const std::string &, const std::string &, char *, size_t) const;
    virtual int listxattr (const std::string &, char *, size_t) const;
    virtual int removexattr (const std::string &, const std::string &);

  protected:
    bool create_directory (const std::string &, mode_t);

    std::string cache_location;
};



#endif
