/***************************************************************************
 *   Copyright (C) 2004 by Petric Frank                                    *
 *   pfrank@gmx.de                                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef __FUSE_BASE_H__
#define __FUSE_BASE_H__

#include <fuse.h>
#include <string>



// forward references
class fuse_callback;



class fuse_base
{
  public:
    fuse_base ();
    virtual ~fuse_base ();

    void run (int, char *[]);

  protected:
    friend class fuse_callback;

    void usage(const std::string &);

    virtual void help_cmdline (const std::string &);
    virtual void help_options ();
    virtual void help_mount_options ();
    
    virtual bool run_prepare (int &, char *[]);
  
    virtual int getattr (const char *, struct stat *);
    virtual int readlink (const char *, char *, size_t);
    virtual int getdir (const char *, fuse_dirh_t, fuse_dirfil_t);
    virtual int mknod (const char *, mode_t, dev_t);
    virtual int mkdir (const char *, mode_t);
    virtual int unlink (const char *);
    virtual int rmdir (const char *);
    virtual int symlink (const char *, const char *);
    virtual int rename (const char *, const char *);
    virtual int link (const char *, const char *);
    virtual int chmod (const char *, mode_t);
    virtual int chown (const char *, uid_t, gid_t);
    virtual int truncate (const char *, off_t);
    virtual int utime (const char *, struct utimbuf *);
    virtual int open (const char *, int);
    virtual int read (const char *, char *, size_t, off_t);
    virtual int write (const char *, const char *, size_t, off_t);
    virtual int statfs (const char *, struct statfs *);
    virtual int flush (const char *);
    virtual int release (const char *, int);
    virtual int fsync (const char *, int);
    virtual int setxattr (const char *, const char *, const char *, size_t, int);
    virtual int getxattr (const char *, const char *, char *, size_t);
    virtual int listxattr (const char *, char *, size_t);
    virtual int removexattr (const char *, const char *);

  private:
    fuse_callback *callback;
};



#endif
