/***************************************************************************
 *   Copyright (C) 2004 by Petric Frank                                    *
 *   pfrank@gmx.de                                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "fuse-callback.h"
#include "fuse-base.h"
#include <errno.h>
#include <fcntl.h>
#include <string.h>


fuse_callback *fuse_callback::self = 0;
fuse_base *fuse_callback::base = 0;

struct fuse_operations fuse_callback::oper =
{
  fuse_callback::getattr,
  fuse_callback::readlink,
  fuse_callback::getdir,
  fuse_callback::mknod,
  fuse_callback::mkdir,
  fuse_callback::unlink,
  fuse_callback::rmdir,
  fuse_callback::symlink,
  fuse_callback::rename,
  fuse_callback::link,
  fuse_callback::chmod,
  fuse_callback::chown,
  fuse_callback::truncate,
  fuse_callback::utime,
  fuse_callback::open,
  fuse_callback::read,
  fuse_callback::write,
  fuse_callback::statfs,
  fuse_callback::flush,
  fuse_callback::release,
  fuse_callback::fsync,
  fuse_callback::setxattr,
  fuse_callback::getxattr,
  fuse_callback::listxattr,
  fuse_callback::removexattr,
};



fuse_callback::fuse_callback ()
{
  base = 0;
}



fuse_callback *fuse_callback::instance ()
{
  if (!self)
    self = new fuse_callback ();

  return self;
}



void fuse_callback::setbase (fuse_base *newbase)
{
  base = newbase;
}



int fuse_callback::getattr (const char *path, struct stat *stbuf)
{
  if (base)
    return base->getattr (path, stbuf);
  
  return -ENOENT;
}



int fuse_callback::readlink (const char *path, char *buf, size_t size)
{
  if (base)
    return base->readlink (path, buf, size);

  return -ENOENT;
}



int fuse_callback::getdir (const char *path, fuse_dirh_t h, fuse_dirfil_t filler)
{
  if (base)
    return base->getdir (path, h, filler);

  return -ENOENT;
}



int fuse_callback::mknod (const char *path, mode_t mode, dev_t rdev)
{
  if (base)
    return base->mknod (path, mode, rdev);

  return -ENOENT;
}



int fuse_callback::mkdir (const char *path, mode_t mode)
{
  if (base)
    return base->mkdir (path, mode);

  return -ENOENT;
}



int fuse_callback::unlink (const char *path)
{
  if (base)
    return base->unlink (path);

  return -ENOENT;
}



int fuse_callback::rmdir (const char *path)
{
  if (base)
    return base->rmdir (path);

  return -ENOENT;
}



int fuse_callback::symlink (const char *path, const char *to)
{
  if (base)
    return base->symlink (path, to);

  return -ENOENT;
}



int fuse_callback::rename (const char *path, const char *to)
{
  if (base)
    return base->rename (path, to);

  return -ENOENT;
}



int fuse_callback::link (const char *path, const char *to)
{
  if (base)
    return base->link (path, to);

  return -ENOENT;
}



int fuse_callback::chmod (const char *path, mode_t mode)
{
  if (base)
    return base->chmod (path, mode);

  return -ENOENT;
}



int fuse_callback::chown (const char *path, uid_t uid, gid_t gid)
{
  if (base)
    return base->chown (path, uid, gid);

  return -ENOENT;
}



int fuse_callback::truncate (const char *path, off_t size)
{
  if (base)
    return base->truncate (path, size);

  return -ENOENT;
}



int fuse_callback::utime (const char *path, struct utimbuf *buf)
{
  if (base)
    return base->utime (path, buf);

  return -ENOENT;
}



int fuse_callback::open (const char *path, int flags)
{
  if (base)
    return base->open (path, flags);
    
  return -ENOENT;
}



int fuse_callback::read (const char *path, char *buf, size_t size, off_t offset)
{
  if (base)
    return base->read (path, buf, size, offset);
    
  return -ENOENT;
}



int fuse_callback::write (const char *path, const char *buf, size_t size, off_t offset)
{
  if (base)
    return base->write (path, buf, size, offset);
    
  return -ENOENT;
}



int fuse_callback::statfs (const char *path, struct statfs *stbuf)
{
  if (base)
    return base->statfs (path, stbuf);
    
  return -ENOENT;
}



int fuse_callback::flush (const char *path)
{
  if (base)
    return base->flush (path);
    
  return -ENOENT;
}



int fuse_callback::release (const char *path, int mode)
{
  if (base)
    return base->release (path, mode);
    
  return -ENOENT;
}



int fuse_callback::fsync (const char *path, int mode)
{
  if (base)
    return base->fsync (path, mode);
    
  return -ENOENT;
}



int fuse_callback::setxattr (const char *path, const char *name, const char *value, size_t size, int flags)
{
  if (base)
    return base->setxattr (path, name, value, size, flags);
    
  return -ENOENT;
}



int fuse_callback::getxattr (const char *path, const char *name, char *value, size_t size)
{
  if (base)
    return base->getxattr (path, name, value, size);
    
  return -ENOENT;
}



int fuse_callback::listxattr (const char *path, char *list, size_t size)
{
  if (base)
    return base->listxattr (path, list, size);
    
  return -ENOENT;
}



int fuse_callback::removexattr (const char *path, const char *name)
{
  if (base)
    return base->removexattr (path, name);
    
  return -ENOENT;
}
