/***************************************************************************
 *   Copyright (C) 2004 by Petric Frank                                    *
 *   pfrank@gmx.de                                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef __VIEW_H__
#define __VIEW_H__

#include <string>
#include "directory.h"



// forward reference
class fs_layer;



class view
{
  public:
    view ();
    ~view ();

    void shutdown ();

    void set_private_layer (fs_layer *);
    void purge_layers ();
    void append_layer (fs_layer *);

    directory *readdir (const std::string &) const;
    int getattr (const std::string &, struct stat *) const;
    int readlink (const std::string &, char *, size_t) const;
    int mknod (const std::string &, mode_t, dev_t);
    int mkdir (const std::string &, mode_t);
    int unlink (const std::string &);
    int rmdir (const std::string &);
    int symlink (const std::string &, const std::string &);
    int rename (const std::string &, const std::string &);
    int link (const std::string &, const std::string &);
    int chmod (const std::string &, mode_t);
    int chown (const std::string &, uid_t, gid_t);
    int truncate (const std::string &, off_t);
    int utime (const std::string &, struct utimbuf *);
    int open (const std::string &, int);
    int read (const std::string &, char *, size_t, off_t) const;
    int write (const std::string &, const char *, size_t, off_t);
    int statfs (const std::string &, struct statfs *) const;
    int flush (const std::string &);
    int release (const std::string &, int);
    int fsync (const std::string &, int);
    int setxattr (const std::string &, const std::string &, const std::string &, size_t, int);
    int getxattr (const std::string &, const std::string &, char *, size_t) const;
    int listxattr (const std::string &, char *, size_t) const;
    int removexattr (const std::string &, const std::string &);

  private:
    int add_item (const std::string &, char *, size_t);
    int handle_response (const std::string &, char *, size_t);

    fs_layer *PrivateLayer;
    typedef std::list<fs_layer *> ListType;
    
    ListType layers;
};



#endif
